#include "owl.h"

void owl_mainwin_init(owl_mainwin *mw) {
  mw->isfull=0;
  mw->lastdisplayed=-1;
}

void owl_mainwin_redisplay(owl_mainwin *mw) {
  owl_message *m;
  int i, j, lines, x,y, savey, topmsg, curmsg, recwinlines;
  WINDOW *recwin;
  owl_messagelist *msglist;

  msglist=owl_global_get_msglist(&g);
  recwin=owl_global_get_curs_recwin(&g);
  topmsg=owl_global_get_topmsg(&g);
  curmsg=owl_global_get_curmsg(&g);

  if (owl_messagelist_get_size(msglist)==0) {
    owl_global_set_topmsg(&g, 0);
    werase(recwin);
    wnoutrefresh(recwin);
    owl_global_set_needrefresh(&g);
    return;
  }

  /* adjust topmsg appropriately */
  /* owl_mainwin_calculate_topmsg(mw); */

  recwinlines=owl_global_get_recwin_lines(&g);
  topmsg=owl_global_get_topmsg(&g);

  werase(recwin);

  /* write the messages out */
  mw->isfull=0;
  j=owl_messagelist_get_size(msglist);
  for (i=topmsg; i<j; i++) {
    if (mw->isfull) break;
    m=owl_messagelist_get_element(msglist, i);

    /* hold on to y in case this is the current message or deleted */
    getyx(recwin, y, x);
    savey=y;

    /* check that we won't fill the window */    
    lines=owl_message_get_numlines(m);
    if (y+lines > recwinlines-3) {
      mw->isfull=1;
      owl_message_curs_waddstr(m,
			       owl_global_get_curs_recwin(&g),
			       0,
			       recwinlines-y,
			       owl_global_get_rightshift(&g),
			       owl_global_get_cols(&g)+owl_global_get_rightshift(&g)-1);

    } else {
      /* print the message */
      owl_message_curs_waddstr(m, owl_global_get_curs_recwin(&g),
			       0,
			       owl_message_get_numlines(m),
			       owl_global_get_rightshift(&g),
			       owl_global_get_cols(&g)+owl_global_get_rightshift(&g)-1);
    }


    /* is it the current message and/or deleted? */
    getyx(recwin, y, x);
    wattron(recwin, A_BOLD);	
    if (owl_global_get_rightshift(&g)==0) {   /* this lame and should be fixed */
      if (m==owl_messagelist_get_element(msglist, curmsg)) {
	wmove(recwin, savey, 0);
	if (!owl_message_is_delete(m)) {
	  waddstr(recwin, "->");
	} else {
	  waddstr(recwin, "-D");
	}
	wmove(recwin, y, x);
      }  else if (owl_message_is_delete(m)) {
	wmove(recwin, savey, 0);
	waddstr(recwin, " D");
	wmove(recwin, y, x);
      }
    }
    wattroff(recwin, A_BOLD);
  }
  mw->lastdisplayed=i-1;

  wnoutrefresh(recwin);
  owl_global_set_needrefresh(&g);
}


int owl_mainwin_is_full(owl_mainwin *mw) {
  if (mw->isfull) return(1);
  return(0);
}

void owl_mainwin_calculate_topmsg(owl_mainwin *mw) {
  /* Figure out if we need to scroll (by chaning topmsg) */
  int recwinlines, y, savey, i, j, topmsg, curmsg, foo;
  owl_messagelist *msglist;

  msglist=owl_global_get_msglist(&g);
  topmsg=owl_global_get_topmsg(&g);
  curmsg=owl_global_get_curmsg(&g);
  recwinlines=owl_global_get_recwin_lines(&g);
  
  /* Find number of lines from top to bottom of curmsg (store in savey) */
  savey=0;
  for (i=topmsg; i<=curmsg; i++) {
    savey+=owl_message_get_numlines(owl_messagelist_get_element(msglist, i));
  }

  /* If our bottom line is less than 1/4 down the screen then scroll up */
  if (savey < (recwinlines / 4)) {
    y=0;
    for (j=curmsg; j>0; j--) {
      foo=owl_message_get_numlines(owl_messagelist_get_element(msglist, j));
      /* will we run the curmsg off the screen? */
      if ((foo+y) >= recwinlines) {
	j++;
	if (j>curmsg) j=curmsg;
	break;
      }
      /* have saved 1/2 the screen space? */
      y+=foo;
      if (y > (recwinlines / 2)) break;
    }
    owl_global_set_topmsg(&g, j);
    return;
  }

  /* If our bottom line is more than 3/4 down the screen then scroll down */
  if (savey > ((recwinlines * 3)/4)) {
    y=0;
    /* count lines until we can save 1/2 the screen size */
    for (j=topmsg; j<curmsg; j++) {
      y+=owl_message_get_numlines(owl_messagelist_get_element(msglist, j));
      if (y > (recwinlines / 2)) break;
    }
    if (j==curmsg) {
      j--;
    }
    owl_global_set_topmsg(&g, j+1);
    return;
  }
}
